(***********************************************************************)
(*                                                                     *)
(*                               Grimage                               *)
(*                                                                     *)
(*                           Graphics image                            *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2014-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: generic_image.mli,v 1.9 2014/02/27 16:45:22 fclement Exp $ *)

(* The generic image format as used in this library.
   This format is used to read and write image files, in various proprietary
   formats. *)

type red = int and green = int and blue = int;;
 (** The RGB components of colors. *)

type width = int and height = int;;
 (** The width and height integer values for graphical objects. *)
type x = int and y = int;;
 (** The x and y integer coordinates in the Graphics window. *)

type pixmap = Graphics.color array array;;
 (** Pixmaps are represented as matrices of colors. *)

type bluemap = pixmap;;
 (** Bluemaps are matrices of integers in the range [0..255]: a bluemap can
   contain the intensity of one color component of all the pixels in a
   pixmap. Hence, a blue map is just a pixmap whose all pixels are indeed
   lesser than 255. As a consequence, if a bluemap is displayed as a pixmap,
   the resulting image is made of various kinds of blue; for the same reason,
   if you want to display a bluemap as an image made of various amounts of its
   original color component (red, green, or blue), you should first convert it
   to a monochrome pixmap using one of [red_pixmap_of_bluemap],
   [blue_pixmap_of_bluemap], or [green_pixmap_of_bluemap].
   Note that a [bluemap] for the [red] channel is a blue image. *)

type red_pixmap = pixmap;;
type green_pixmap = pixmap;;
type blue_pixmap = pixmap;;
 (** The bluemap corresponding notion for red, green and blue colors:
   the corresponding [pixmap] is monocolor of the corresponding color.
   Note that the [blue_pixmap] corresponding to the [blue] channel of an image
   is a [bluemap]. *)

type file_name = string;;
 (** File names are represented as strings. *)

type rgb = {r : red; g : green; b : blue};;
 (** Colormap items : R, G, B values. *)

type colormap = rgb array;;
 (** Color maps. *)

type bpp8descr = {
 data : string;
 colormap : colormap;
 transparent : Graphics.color;
};;
 (** 8 bits per pixels images: each pixel of the image is represented by
    its color, a byte of the [data] field string, which is an index
    to the color map vector, that contains the set of colors used in the
    image. *)

type bitmap =
   | Bpp8 of bpp8descr
   | Bpp24 of string;;
  (** Bitmap values are mainly big strings, that can be of two flavours
     - 8 bits per pixels bitmaps: these one use a color map, each byte
       of the string is an index in the color map, that encodes the color
       of the corresponding pixel.
     - 24 bits per pixel or true color: no color map, the color of each
       pixel is given as its R, G, B values. Hence, bytes in the string
       must be taken 3 by 3 to give RGB values of pixels. *)

type image = {
 bitmap : bitmap;
 width : width;
 height : height;
};;
  (** The Generic image type used in this library. *)
