(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uhomogeneous_tuple.ml,v 1.22 2014-02-26 00:16:54 fclement Exp $ *)

module Nat = Unatural;;

let ht_qual mod_name =
  Upervasives.qualify (Upervasives.qualify "Uhomogeneous_tuple" mod_name)
;;

let s_failwith = Upervasives.failwith_qual (ht_qual "Single_structure")
and d_failwith = Upervasives.failwith_qual (ht_qual "Double_structure")
and t_failwith = Upervasives.failwith_qual (ht_qual "Triple_structure")
and v_failwith = Upervasives.failwith_qual (ht_qual "Vector_structure")
;;

module Make =
  functor (S : Uhomogeneous_tuple_spec.STRUCTURE) ->
struct
  include S;;
  
  let make x = init (fun _ -> x);;
  
  let same equal tup1 tup2 =
    fold_left2 (fun b x1 x2 -> b && equal x1 x2) true tup1 tup2
  ;;
  
  let singleton i0 xi0 default =
    init (fun i -> if i = i0 then xi0 else default)
  ;;
  
  let of_array a = of_list (Array.to_list a);;
  let to_array tup = Array.of_list (to_list tup);;
  
  let map f tup = init (fun i -> f (get tup i));;
  let map_i f tup = init (fun i -> f i (get tup i));;
  let map2 f tup1 tup2 = init (fun i -> f (get tup1 i) (get tup2 i));;
  let map3 f tup1 tup2 tup3 =
    init (fun i -> f (get tup1 i) (get tup2 i) (get tup3 i))
  ;;
  let map4 f tup1 tup2 tup3 tup4 =
    init (fun i -> f (get tup1 i) (get tup2 i) (get tup3 i) (get tup4 i))
  ;;
  
  let map_rails ftup tup = init (fun i -> (get ftup i) (get tup i));;
  let map_rails2 ftup tup1 tup2 =
    init (fun i -> (get ftup i) (get tup1 i) (get tup2 i))
  ;;
  let iter_rails2 ftup tup1 = iter_rails (map_rails ftup tup1);;
  let iter_rails3 ftup tup1 tup2 = iter_rails (map_rails2 ftup tup1 tup2);;
  
  let components f = S.init (fun i pt -> S.get (f pt) i);;
  
  let array_components a = init (fun i -> Array.map (fun aj -> get aj i) a);;
  
  let components_map f comps = map (Upervasives.compose f) comps;;
  
  let split tup =
    let tup1 = map fst tup in
    let tup2 = map snd tup in
    (tup1, tup2)
  ;;
  
  let combine comps = (fun pt -> S.init (fun i -> S.get comps i pt));;
end
;;

(* Single. *)

module Single_structure =
struct
  type 'a t = 'a;;
  
  let get_dim () = Nat._1_;;
  
  let print = Upervasives.id;;
  
  let get x0 i =
    match Nat.to_int i with
    | 0 -> x0
    | _ -> assert false
  ;;
  
  let init f = f Nat._0_;;
  
  let of_list = function
    | x :: [] -> x
    | _ -> s_failwith "of_list" "wrong length"
  ;;
  let to_list x = [ x; ];;
  
  let of_triple (x0, x1, x2) =
    if x0 = x1 && x1 = x2 && x2 = x0 then x0 else
    s_failwith "of_triple" "triple is not on the principal diagonal"
  ;;
  let to_triple x = (x, x, x);;
  
  let iter = Upervasives.id;;
  let iter_i f = f Nat._0_;;
  let iter_rails = Upervasives.id;;
  let fold_left = Upervasives.id;;
  let fold_left_i f accu = f accu Nat._0_;;
  let iter2 = Upervasives.id;;
  let fold_left2 = Upervasives.id;;
  let iter3 = Upervasives.id;;
  let iter4 = Upervasives.id;;
end
;;

module Single = Make (Single_structure);;

(* Double. *)

module Double_structure =
struct
  type 'a t = 'a * 'a;;
  
  let get_dim () = Nat._2_;;
  
  let print print_scalar ppf (x0, x1) =
    Format.fprintf ppf "@[<hv 2>(%a,@ %a)@]"
      print_scalar x0
      print_scalar x1
  ;;
  
  let get (x0, x1) i =
    match Nat.to_int i with
    | 0 -> x0
    | 1 -> x1
    | _ -> assert false
  ;;
  
  let init f = (f Nat._0_, f Nat._1_);;
  
  let of_list = function
    | x1 :: x2 :: [] -> (x1, x2)
    | _ -> d_failwith "of_list" "wrong length"
  ;;
  let to_list (x0, x1) = [ x0; x1; ];;
  
  let of_triple _ = d_failwith "of_triple" "wrong dimension";;
  let to_triple _ = d_failwith "of_triple" "wrong dimension";;
  
  let iter f (x0, x1) = f x0; f x1;;
  let iter_i f (x0, x1) = f Nat._0_ x0; f Nat._1_ x1;;
  let iter_rails (f0, f1) (x0, x1) = f0 x0; f1 x1;;
  let fold_left f s0 (x0, x1) = f (f s0 x0) x1;;
  let fold_left_i f s0 (x0, x1) = f (f s0 Nat._0_ x0) Nat._1_ x1;;
  let iter2 f (x0, x1) (y0, y1) = f x0 y0; f x1 y1;;
  let fold_left2 f s0 (x0, x1) (y0, y1) = f (f s0 x0 y0) x1 y1;;
  let iter3 f (x0, x1) (y0, y1) (z0, z1) = f x0 y0 z0; f x1 y1 z1;;
  let iter4 f (x0, x1) (y0, y1) (z0, z1) (t0, t1) =
    f x0 y0 z0 t0; f x1 y1 z1 t1
  ;;
end
;;

module Double = Make (Double_structure);;

(* Triple. *)

module Triple_structure =
struct
  type 'a t = 'a * 'a * 'a;;
  
  let get_dim () = Nat._3_;;
  
  let print print_scalar ppf (x0, x1, x2) =
    Format.fprintf ppf "@[<hv 2>(%a,@ %a,@ %a)@]"
      print_scalar x0
      print_scalar x1
      print_scalar x2
  ;;
  
  let get (x0, x1, x2) i =
    match Nat.to_int i with
    | 0 -> x0
    | 1 -> x1
    | 2 -> x2
    | _ -> assert false
  ;;
  
  let init f = (f Nat._0_, f Nat._1_, f Nat._2_);;
  
  let of_list = function
    | x1 :: x2 :: x3 :: [] -> (x1, x2, x3)
    | _ -> t_failwith "of_list" "wrong length"
  ;;
  let to_list (x0, x1, x2) = [ x0; x1; x2; ];;
  
  let of_triple = Upervasives.id;;
  let to_triple = Upervasives.id;;
  
  let iter f (x0, x1, x2) = f x0; f x1; f x2;;
  let iter_i f (x0, x1, x2) = f Nat._0_ x0; f Nat._1_ x1; f Nat._2_ x2;;
  let iter_rails (f0, f1, f2) (x0, x1, x2) = f0 x0; f1 x1; f2 x2;;
  let fold_left f s0 (x0, x1, x2) = f (f (f s0 x0) x1) x2;;
  let fold_left_i f s0 (x0, x1, x2) =
    f (f (f s0 Nat._0_ x0) Nat._1_ x1) Nat._2_ x2
  ;;
  let iter2 f (x0, x1, x2) (y0, y1, y2) = f x0 y0; f x1 y1; f x2 y2;;
  let fold_left2 f s0 (x0, x1, x2) (y0, y1, y2) =
    f (f (f s0 x0 y0) x1 y1) x2 y2
  ;;
  let iter3 f (x0, x1, x2) (y0, y1, y2) (z0, z1, z2) =
    f x0 y0 z0; f x1 y1 z1; f x2 y2 z2
  ;;
  let iter4 f (x0, x1, x2) (y0, y1, y2) (z0, z1, z2) (t0, t1, t2) =
    f x0 y0 z0 t0; f x1 y1 z1 t1; f x2 y2 z2 t2
  ;;
end
;;

module Triple = Make (Triple_structure);;

(* Vector. *)

open Uvector_spec;;

module Vector_structure =
  functor (V : VECTOR) ->
struct
  include V;;
  
  let of_triple t =
    match Nat.to_int (get_dim ()) with
    | 1 -> of_array [| (Single_structure.of_triple t); |]
    | 3 -> of_list (Triple_structure.to_list t)
    | _ -> v_failwith "of_triple" "wrong dimension"
  ;;
  let to_triple v =
    match Nat.to_int (get_dim ()) with
    | 1 -> Single_structure.to_triple (get v Nat._0_)
    | 3 -> Triple_structure.of_list (to_list v)
    | _ -> v_failwith "to_triple" "wrong dimension"
  ;;
end
;;

module Vector = functor (V : VECTOR) -> Make (Vector_structure (V));;
